
--------------------------------------------------------------------------------------------------------------------------
--[[

TilingBox v.1.0 by Galuhad

Ths class provides authors with a TilingBox similar to the listbox, which automatically wraps controls to a new line when
the edge of the box has already been reached. TilingBox is based off Turbine.UI.Control - so most standard methods will apply.

Usage:

someControl = TilingBox();				-- Creates a new instance of type Tiling Box
someControl:SetParent(parentControl);	--
someControl:SetPosition(0,0);
someControl:SetSize(300,200);

someControl:AddItem(control);		-- Functions the same as ListBox:AddItem()
someControl:RemoveItem(control);	-- Functions the same as ListBox:RemoveItem()
someControl:ContainsItem(control);	-- Checks if the box contains said item, returns boolean
someControl:ClearItems();			-- Clears all items from the box
someControl:GetItemCount();			-- Returns the number of items in the box
someControl:GetItem(index);			-- Returns the control for the item at the given index

Events:
someControl.ItemAdded = function (sender,args)		-- the event sender, the event arguments
someControl.ItemRemoved = function (sender,args)	-- the event sender, the event arguments
someControl.ChangedSize = function (sender,args)	-- the event sender, the event arguments	-- this is because SizeChanged is already used

The following arguments are passed in ItemAdded and ItemRemoved
args.index		-- the index of the item within the box
args.item		-- the item/control that is being added/removed

--]]
---------------------------------------------------------------------------------------------------------------------------


TilingBox = {};
_mtTilingBox = {};

function TilingBox.Constructor()

	local BoxContainer = Turbine.UI.Control();
	BoxContainer:SetSize(200,200);

	BoxContainer.SizeChanged = function(sender,args)
		BoxContainer:RepositionControls()
		if type(BoxContainer.ChangedSize) == 'function' then BoxContainer.ChangedSize(sender,args) end;
	end

	function BoxContainer:RepositionControls()

		local BoxControls = BoxContainer:GetControls();
		local ControlCount = BoxControls:GetCount();

		for i=1, ControlCount do
			local Item = BoxControls:Get(i);
			if i > 1 then
				local previousItem = BoxControls:Get(i-1);
				local itemRight = previousItem:GetLeft()+previousItem:GetWidth()+Item:GetWidth();
				if itemRight > BoxContainer:GetWidth() then
					Item:SetLeft(0);
					Item:SetTop(previousItem:GetTop()+previousItem:GetHeight());
				else
					Item:SetLeft(previousItem:GetLeft()+previousItem:GetWidth());
					Item:SetTop(previousItem:GetTop());
				end
			else
				Item:SetPosition(0,0);
			end
		end
	end


	function BoxContainer:AddItem(Item)
		if Item == nil then return end;
		local BoxControls = BoxContainer:GetControls();
		if BoxControls:Contains(Item) then return end;
		BoxControls:Add(Item);
		BoxContainer:RepositionControls();
		BoxContainer.ItemAdded(BoxContainer,{["item"]=Item;["index"]=BoxControls:IndexOf(Item);});
	end


	function BoxContainer:ContainsItem(Item)
		if Item == nil then return end;
		local BoxControls = BoxContainer:GetControls();
		return BoxControls:Contains(Item);
	end


	function BoxContainer:RemoveItem(Item)
		if Item == nil then return end;
		local BoxControls = BoxContainer:GetControls();
		if BoxControls:Contains(Item) then
			local oldIndex = BoxControls:IndexOf(Item);
			BoxControls:Remove(Item);
			BoxContainer:RepositionControls();
			BoxContainer.ItemRemoved(BoxContainer,{["item"]=Item;["index"]=oldIndex;});
		end
	end


	function BoxContainer:ClearItems()
		local BoxControls = BoxContainer:GetControls();
		BoxControls:Clear();
		BoxContainer:RepositionControls();
	end


	function BoxContainer:GetItemCount()
		local BoxControls = BoxContainer:GetControls();
		return BoxControls:GetCount();
	end


	function BoxContainer:GetItem(Index)
		if Index == nil then return end;
		local BoxControls = BoxContainer:GetControls();
		return BoxControls:Get(Index);
	end

	BoxContainer.ItemAdded = function () end;
	BoxContainer.ItemRemoved = function () end;
	BoxContainer.ChangedSize = function () end;

	return BoxContainer;

end


function _mtTilingBox.__call()
    return TilingBox.Constructor();
end


TilingBox.__index = TilingBox;
TilingBox.__newindex = TilingBox;

setmetatable(TilingBox, _mtTilingBox);
